package com.lifepay.pinpaddemo.activities;

import android.content.DialogInterface;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;

import com.lifepay.pinpaddemo.R;
import com.lifepay.pinpaddemo.adapter.TransactionListAdapter;
import com.lifepay.pinpaddemo.activities.base.BaseFragmentActivity;
import com.lifepay.pinpaddemo.activities.base.FBaseMenuBar;
import com.lifepay.pinpaddemo.controller.TransactionsListController;
import com.lifepay.pinpaddemo.utils.ActivityUtils;
import com.lifepay.pinpaddemo.utils.UIUtilsHelper;

import org.androidannotations.annotations.AfterViews;
import org.androidannotations.annotations.EActivity;
import org.androidannotations.annotations.FragmentById;
import org.androidannotations.annotations.InstanceState;
import org.androidannotations.annotations.ViewById;

@EActivity(R.layout.transaction_list)
public class TransactionListActivity extends BaseFragmentActivity {

    @FragmentById
    protected FBaseMenuBar fMenuBar;

    @InstanceState
    protected boolean initFlag;
    private TransactionListAdapter adapter;
    @ViewById
    protected RecyclerView rvTransactionList;
    @ViewById
    protected SwipeRefreshLayout swipeRefreshLayout;
    private LinearLayoutManager layoutManager;
    private int pastVisiblesItems, visibleItemCount, totalItemCount;

    private TransactionsListController transactionsController;

    @AfterViews
    public void init() {
        fMenuBar.setIntf(new FBaseMenuBar.FBaseMenuBarInterface() {
            @Override
            public void onButtonClick(int id) {
                switch (id) {
                    case R.id.btnBack:
                        back();
                        break;
                }
            }
        });

        transactionsController = new TransactionsListController(context, new TransactionsListController.TransactionListModelIntf() {
            @Override
            public void authFailed() {
                UIUtilsHelper.showAlert(context, "Ошибка авторизации. Неправильный логин и пароль.",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog,
                                                int which) {
                                back();
                            }
                        });
            }

            @Override
            public void startLoading() {
                if (transactionsController.getOffset() == 0)
                    showWaitingDialog("Подождите", false);
                if (swipeRefreshLayout != null)
                    swipeRefreshLayout.setRefreshing(true);
            }

            @Override
            public void loaded(boolean isSuccess, String errorMessage) {
                hideWaitingDialog();
                if (swipeRefreshLayout != null)
                    swipeRefreshLayout.setRefreshing(false);
                if (isSuccess) {
                    adapter.refreshList(transactionsController.getTransactions());
                } else {
                    UIUtilsHelper.showAlert(context, errorMessage,
                            new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog,
                                                    int which) {
                                    back();
                                }
                            });
                }
            }
        });

        rvTransactionList.setHasFixedSize(true);

        layoutManager = new LinearLayoutManager(context);
        rvTransactionList.setLayoutManager(layoutManager);

        adapter = new TransactionListAdapter(transactionsController.getTransactions(), context);
        rvTransactionList.setAdapter(adapter);

        swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                transactionsController.refreshTransations();
            }
        });
        rvTransactionList.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                visibleItemCount = layoutManager.getChildCount();
                totalItemCount = layoutManager.getItemCount();
                pastVisiblesItems = layoutManager.findFirstVisibleItemPosition();
                transactionsController.setLastVisiblesItemPosition(pastVisiblesItems);

                if (!swipeRefreshLayout.isRefreshing()) {
                    if ((visibleItemCount + pastVisiblesItems) >= totalItemCount) {
                        transactionsController.loadMore();
                    }
                }
            }
        });

        rvTransactionList.scrollToPosition(transactionsController.getLastVisiblesItemPosition());
    }

    @Override
    public void back() {
        ActivityUtils.startActivity(this, MainActivity_.class);
        transactionsController.destroy();
    }
}
