package com.lifepay.pinpaddemo.activities.pinpad.firmware;

import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Configuration;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.lifepay.lib.beans.TerminalType;
import com.lifepay.lib.models.PinpadFabric;
import com.lifepay.lib.reader.utils.BluetoothDeviceSearcher;
import com.lifepay.lib.reader.utils.PinpadBluetoothDevice;
import com.lifepay.lib.reader.utils.qpos.QPOSFirmwareLoader;
import com.lifepay.lib.rest.params.AuthParams;
import com.lifepay.pinpaddemo.R;
import com.lifepay.pinpaddemo.TestData;
import com.lifepay.pinpaddemo.activities.MainActivity_;
import com.lifepay.pinpaddemo.activities.base.BaseFragmentActivity;
import com.lifepay.pinpaddemo.utils.ActivityUtils;
import com.lifepay.pinpaddemo.utils.UIUtilsHelper;

import org.androidannotations.annotations.AfterViews;
import org.androidannotations.annotations.Click;
import org.androidannotations.annotations.EActivity;
import org.androidannotations.annotations.FragmentById;
import org.androidannotations.annotations.UiThread;
import org.androidannotations.annotations.ViewById;

import java.util.ArrayList;
import java.util.List;

@EActivity(R.layout.qpos_loadfirmware_layout)
public class QPOSLoadFirmwareActivity extends BaseFragmentActivity {
    @FragmentById
    protected FQPOSLoadFirmwareBluetoothSelect fBluetoothSelect;
    @ViewById
    protected TextView tvConnectionError;
    @ViewById
    protected Button btnStart;
    @ViewById
    protected Button btnRepeat;
    @ViewById
    protected Button btnSuccessClose;
    @ViewById
    protected ImageView ivClose;
    @ViewById
    protected TextView tvSuccess;
    @ViewById
    protected TextView tvStatus;
    @ViewById
    protected TextView tvError;
    @ViewById
    protected TextView tvHelp;
    @ViewById
    protected TextView tvStatusHelp;

    @ViewById
    protected RelativeLayout containerProcessFirmware;
    @ViewById
    protected LinearLayout containerHelp;
    @ViewById
    protected RelativeLayout containerError;
    @ViewById
    protected RelativeLayout containerSuccess;

    protected BroadcastReceiver networkReceiver;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // TODO Auto-generated method stub
        super.onCreate(savedInstanceState);

        if(context==null)
            return;

        networkReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {
                if (intent.getExtras() != null) {
                    NetworkInfo ni = (NetworkInfo) intent.getExtras().get(ConnectivityManager.EXTRA_NETWORK_INFO);
                    if (ni != null && ni.getState() == NetworkInfo.State.CONNECTED) {
                        onNetworkConnected();
                    }
                }
                if (intent.getExtras().getBoolean(ConnectivityManager.EXTRA_NO_CONNECTIVITY, Boolean.FALSE)) {
                    onNetworkDisconnected();
                }
            }
        };

        registerReceiver(networkReceiver, new IntentFilter("android.net.conn.CONNECTIVITY_CHANGE"));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (networkReceiver != null)
            unregisterReceiver(networkReceiver);
    }

    protected void onNetworkConnected() {
        if (tvConnectionError != null)
            tvConnectionError.setVisibility(View.GONE);
    }

    protected void onNetworkDisconnected() {
        tvConnectionError.setVisibility(View.VISIBLE);
    }

    protected enum FirmwareStage {
        HELP, SELECT_DEVICE, PROCESS, SUCCESS, ERROR
    }

    private boolean isTablet(Context context) {
        return (context.getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_LARGE;
    }

    @AfterViews
    public void init() {
        if (context == null)
            return;

        String deviceType = isTablet(context) ? "планшетом" : "телефоном";
        tvHelp.setText("Для обновления терминал обязательно должен быть подключен к зарядному устройству.\nОбновление занимает от 5 до 8 минут.\nВо время обновления нельзя производить никаких действий с " + deviceType + " и терминалом - это может привести к неисправности терминала.");
        tvStatusHelp.setText("Обновление занимает от 5 до 8 минут. Не производите никаких действий с " + deviceType + " и терминалом, не отключайте терминал от зарядного устройства - это может привести к неисправности терминала.");

        PinpadFabric.setTerminalType(TerminalType.DSPREAD_QPOS);
        PinpadFabric.getDeviceSearcher().updateContextAndIntf(context, new BluetoothDeviceSearcher.BluetoothDeviceSearcherIntf() {
            @Override
            public void update() {
                updateBluetoothList();
            }
        });

        if (!isTablet(context)) {
            tvHelp.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 18);
        }

        fBluetoothSelect.setIntf(new FQPOSLoadFirmwareBluetoothSelect.FQPOSLoadFirmwareBluetoothSelectIntf() {
            @Override
            public void select(int index) {
                fBluetoothSelect.hide();
                BluetoothDevice device = PinpadFabric.getDeviceSearcher().getDevices().get(index);
                PinpadFabric.getDeviceSearcher().stop();

                QPOSFirmwareLoader.getInstance().selectDevice(new PinpadBluetoothDevice(device));
            }

            @Override
            public void update() {
                updateBluetoothList();
            }
        });
        fBluetoothSelect.hide();

        showStatus("", FirmwareStage.HELP);
    }

    @Click(R.id.ivClose)
    public void onCloseBtnClicked() {
        back();
    }

    @Click(R.id.btnStart)
    public void onStartBtnClicked() {
        startProcess();
    }

    @Click(R.id.btnRepeat)
    public void onRepeatBtnClicked() {
        startProcess();
    }

    @Click(R.id.btnSuccessClose)
    public void onSuccessCloseBtnClicked() {
        back();
    }

    private void startProcess() {
        //нужно подставить логин пароль от своего аккаунта
        AuthParams authParams = new AuthParams(TestData.LOGIN, TestData.PASSWORD);

        QPOSFirmwareLoader.getInstance().start(context, authParams, new QPOSFirmwareLoader.QPOSFirmwareLoaderCallback() {
            @Override
            public void onDeviceSelection() {
                showStatus("", FirmwareStage.SELECT_DEVICE);
                fBluetoothSelect.show();
                fBluetoothSelect.updateDevicesList(new ArrayList<String>());
                PinpadFabric.getDeviceSearcher().start();
            }

            @Override
            public void onConnecting(PinpadBluetoothDevice device) {
                showStatus("Подключение\n"+device.name, FirmwareStage.PROCESS);
            }

            @Override
            public void onProcessStarted() {
                showStatus("Подождите...", FirmwareStage.PROCESS);
            }

            @Override
            public void onFirmwareLoading(int loadingPercent) {
                showStatus("Обновление: "+loadingPercent+"%", FirmwareStage.PROCESS);
            }

            @Override
            public void onFirmwareLoaded(String msg) {
                showStatus(msg, FirmwareStage.SUCCESS);
            }

            @Override
            public void onFail(String msg) {
                showStatus(msg, FirmwareStage.ERROR);
            }

            @Override
            public void onAuthError(String msg) {
                showAuthError(msg);
            }
        });
    }

    @UiThread
    public void updateBluetoothList() {
        List<BluetoothDevice> devices = PinpadFabric.getDeviceSearcher().getDevices();
        List<String> list = new ArrayList<>();
        for(BluetoothDevice bt : devices) {
            list.add(bt.getName());
        }
        fBluetoothSelect.updateDevicesList(list);
    }

    @UiThread
    public void showStatus(String msg, FirmwareStage stage) {
        containerHelp.setVisibility(View.GONE);
        containerProcessFirmware.setVisibility(View.GONE);
        containerSuccess.setVisibility(View.GONE);
        containerError.setVisibility(View.GONE);
        ivClose.setVisibility(View.GONE);

        switch (stage) {
            case HELP:
                containerHelp.setVisibility(View.VISIBLE);
                ivClose.setVisibility(View.VISIBLE);
                break;
            case SELECT_DEVICE:
                ivClose.setVisibility(View.VISIBLE);
                break;
            case PROCESS:
                containerProcessFirmware.setVisibility(View.VISIBLE);
                tvStatus.setText(msg);
                break;
            case SUCCESS:
                containerSuccess.setVisibility(View.VISIBLE);
                tvSuccess.setText(msg);
                break;
            case ERROR:
                ivClose.setVisibility(View.VISIBLE);
                containerError.setVisibility(View.VISIBLE);
                tvError.setText(msg);
                break;
        }
    }

    @UiThread
    public void showAuthError(String msg) {
        showStatus("", FirmwareStage.PROCESS);
        UIUtilsHelper.showErrorMessage(context, "Неправильный логин или пароль", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                forceBack();
            }
        });
    }

    @Override
    public void back() {
        if (QPOSFirmwareLoader.getInstance().stopIfPossible()) {
            forceBack();
        }
    }

    private void forceBack() {
        PinpadFabric.getDeviceSearcher().stop();
        ActivityUtils.startActivity(this, MainActivity_.class);
        finish();
    }
}
