package com.lifepay.pinpaddemo.adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.lifepay.lib.beans.TransactionEx;
import com.lifepay.pinpaddemo.R;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;

public class TransactionListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private String REFUND_TYPE = "2";

    private static final int ITEM_VIEW_TYPE_HEADER = 0;
    private static final int ITEM_VIEW_TYPE_PAYMENT = 1;
    private List<TransactionEx> transactions;
    static DecimalFormat priceFormatter = new DecimalFormat("###,###,###,##0.00");
    static DecimalFormatSymbols otherSymbols = new DecimalFormatSymbols(Locale.getDefault());
    private Context context;

    public TransactionListAdapter(List<TransactionEx> payments, Context context) {
        this.context = context;
        this.transactions = payments;

        otherSymbols.setGroupingSeparator(' ');
        otherSymbols.setDecimalSeparator(',');
        priceFormatter.setDecimalFormatSymbols(otherSymbols);
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (viewType == ITEM_VIEW_TYPE_HEADER) {
            View view = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.item_transaction_list_header_layout, parent, false);
            return new HeaderViewHolder(view);
        }
        View view = LayoutInflater.from(parent.getContext()).inflate(
                        R.layout.item_transaction_list_layout, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        switch (getItemViewType(position)) {
            case ITEM_VIEW_TYPE_HEADER:
                ((HeaderViewHolder) holder).bind(transactions.get(position), position);
                break;
            case ITEM_VIEW_TYPE_PAYMENT:
                ((ViewHolder) holder).bind(transactions.get(position), position);
                break;
        }
    }

    @Override
    public int getItemViewType(int position) {
        return transactions.get(position).getGroup() != null ? ITEM_VIEW_TYPE_HEADER : ITEM_VIEW_TYPE_PAYMENT;
    }

    @Override
    public int getItemCount() {
        if (transactions == null)
            return 0;
        else
            return transactions.size();
    }

    public void refreshList(List<TransactionEx> payments) {
        this.transactions = payments;
        notifyDataSetChanged();
    }

    class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        @BindView(R.id.tvTime)
        TextView tvTime;
        @BindView(R.id.tvAmount)
        TextView tvAmount;
        @BindView(R.id.tvDescription)
        TextView tvDescription;
        @BindView(R.id.tvNumber)
        TextView tvNumber;
        @BindView(R.id.tvCancelled)
        TextView tvCancelled;

        View view;

        public ViewHolder(View view) {
            super(view);
            this.view = view;
            // Inject views
            ButterKnife.bind(this, view);

            view.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
        }

        public void bind(TransactionEx transaction, int position) {
            tvTime.setText(getDateTime(transaction.getCreated()));
            tvDescription.setText(transaction.getDescription());
            tvNumber.setText(transaction.getFormatNumber());

            String amount = priceFormatter.format(Double.valueOf(transaction.getAmount()));
            if (transaction.getType().equals(REFUND_TYPE)) {
                tvAmount.setTextColor(context.getResources().getColor(R.color.blue));
                tvAmount.setText(String.format(Locale.getDefault(), "-%s", amount));
            } else {
                tvAmount.setTextColor(context.getResources().getColor(R.color.textColor));
                tvAmount.setText(amount);
            }

            //Если процессинг поддеживает отмену транзакций, то если через sdk производится возврат день в день,
            //то отдельной операцией возврат не придет, т.к. будет произведена отмена транзакции, а не возврат.
            //В этом случае getCancelled() будет равно "1".
            boolean cancelled = transaction.getCancelled() != null && transaction.getCancelled().equals("1");
            tvCancelled.setVisibility(cancelled? View.VISIBLE : View.GONE);
        }
    }

    private String getDateTime(String datetime) {
        try {
            SimpleDateFormat dtIn = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss", Locale.ENGLISH);
            SimpleDateFormat dtOut = new SimpleDateFormat("HH:mm", Locale.ENGLISH);
            return dtOut.format(dtIn.parse(datetime));
        } catch (Throwable t) {

        }
        return "";
    }

    class HeaderViewHolder extends RecyclerView.ViewHolder {
        @BindView(R.id.tvTotal)
        TextView tvTotal;
        @BindView(R.id.tvDate)
        TextView tvDate;
        @BindView(R.id.llDate)
        LinearLayout llDate;
        @BindView(R.id.flSpace)
        FrameLayout flSpace;

        View view;

        public HeaderViewHolder(View view) {
            super(view);
            this.view = view;
            ButterKnife.bind(this, view);
        }

        public void bind(TransactionEx transaction, int position) {
            tvTotal.setVisibility(View.GONE);
            llDate.setVisibility(View.GONE);
            flSpace.setVisibility(View.GONE);

            if (transaction.getGroup().equals("dateinfo")) {
                llDate.setVisibility(View.VISIBLE);
                tvDate.setText(transaction.getTitle());
                if (position > 0) {
                    flSpace.setVisibility(View.VISIBLE);
                }
            } else {
                tvTotal.setVisibility(View.VISIBLE);
                String title = transaction.getTitle();
                if (title.length() > 3 && title.substring(title.length() - 2).equals("р.")) {
                    title = title.substring(0, title.length() - 3);
                }
                tvTotal.setText(title);
            }
        }
    }
}