package com.lifepay.pinpaddemo.controller;

import android.content.Context;

import com.lifepay.lib.LifePayApi;
import com.lifepay.lib.LifePayException;
import com.lifepay.lib.beans.TransactionEx;
import com.lifepay.lib.models.TransactionsListModel;
import com.lifepay.lib.rest.params.GetTransactionsListExParams;
import com.lifepay.pinpaddemo.TestData;

import java.util.ArrayList;
import java.util.List;

public class TransactionsListController {
    protected ArrayList<TransactionEx> transactionsList = new ArrayList<>();
    protected Context context;

    private int offset = 0;
    private int len = 30;

    int lastVisiblesItemPosition = -1;
    long timeLastUpdate = 0;

    private boolean isLoading = false;
    private boolean isNoMoreTransactions = false;
    private TransactionListModelIntf intf;

    public TransactionsListController(Context context, TransactionListModelIntf intf) {
        this.context = context;
        this.intf = intf;
    }

    public int getOffset() {
        return offset;
    }

    public int getLastVisiblesItemPosition() {
        return transactionsList.size() > lastVisiblesItemPosition ? lastVisiblesItemPosition : -1;
    }

    public void setLastVisiblesItemPosition(int indx) {
        lastVisiblesItemPosition = indx;
    }

    public ArrayList<TransactionEx> getTransactions() {
        if (transactionsList.size() == 0 && System.currentTimeMillis() - timeLastUpdate > 1000) {
            timeLastUpdate = System.currentTimeMillis();
            loadMore();
        }
        return transactionsList;
    }

    public void refreshTransations() {
        if (!isLoading) {
            clear();
            loadMore();
        }
    }

    public void destroy() {
        intf = null;
        clear();
    }

    public void loadMore() {
        if (isLoading)
            return;
        if (isNoMoreTransactions)
            onFinishLoading(true, "");
        else {
            isLoading = true;
            try {
                LifePayApi.getInstance().getTransactionsEx(context, getArgs(), getCallback());
            } catch (LifePayException e) {
                onFinishLoading(false, e.getMessage());
            }
        }
    }

    private void clear() {
        isLoading = false;
        isNoMoreTransactions = false;
        offset = 0;
        lastVisiblesItemPosition = 0;
        transactionsList.clear();
    }

    private void onFinishLoading(boolean success, String message) {
        isLoading = false;
        if (intf != null) intf.loaded(success, message);
    }

    private void onStartLoading() {
        isLoading = true;
        if (intf != null) intf.startLoading();
    }

    private GetTransactionsListExParams getArgs() {
        return new GetTransactionsListExParams(
                TestData.LOGIN, TestData.PASSWORD, offset, len, false, null);
    }

    private TransactionsListModel.TransactionsListExCallback getCallback() {
        return new TransactionsListModel.TransactionsListExCallback() {

            @Override
            public void transactionError(String errorMessage) {
                onFinishLoading(false, errorMessage);
            }

            @Override
            public void onSuccess(List<TransactionEx> trList) {
                //смотрим, нужно ли еще пытаться загружать транзакции.
                //(осталось ли что-то еще на сервере, или уже все загрузили.)
                if (trList.size() < len) {
                    isNoMoreTransactions = true;
                }
                offset += trList.size();

                //добавляем в список дополнительные поля с датой для группировки списка по дням.
                //это делать не обязательно разумеется.
                String lastDate = "";
                if (transactionsList.size() != 0) {
                    int i = transactionsList.size() - 1;
                    while (transactionsList.get(i).getTitle() != null) {
                        i--;
                    }
                    lastDate = transactionsList.get(i).getDate();
                }
                for (TransactionEx op : trList) {
                    if (op.getGroup() == null && !op.getDate().equals(lastDate)) {
                        TransactionEx t = new TransactionEx();
                        t.setGroup("dateinfo");
                        t.setTitle(op.getDate());
                        transactionsList.add(t);
                        lastDate = op.getDate();
                    }
                    transactionsList.add(op);
                }

                //кидаем колбек успеха в UI
                onFinishLoading(true, "");
            }

            @Override
            public void onTransactionSended() {
                onStartLoading();
            }

            @Override
            public void onAuthError(String error) {
                onFinishLoading(false, error);

                if (intf != null) intf.authFailed();
            }
        };
    }

    public interface TransactionListModelIntf {
        void authFailed();
        void startLoading();
        void loaded(boolean isSuccess, String errorMessage);
    }
}
